import numpy as np
from scipy.optimize import least_squares
import matplotlib.pyplot as plt
import Tunneling as tun
plt.style.use("Figures.mplstyle")

def simulatelnKIEs(Ts, deltaZPE, deltaG, height, freq):
	barrier_H = tun.Eckart(height, deltaG, freq)
	barrier_D = tun.Eckart(height+deltaZPE, deltaG, freq*np.sqrt(1/2))
	lnKIE_sim = []
	for T in Ts:
		lnKIE_sim.append(np.log(barrier_H.transmission(T,1)) - np.log(barrier_D.transmission(T,2)))
	return np.array(lnKIE_sim)

def fitKIEs(Ts, KIEs, deltaG = -10, initGuess = [12, 2000], semiclassicals = [0, 0]):
	target = np.log(KIEs) - semiclassicals[0]*503/Ts - semiclassicals[1]
	def resids(params):
		return target - simulatelnKIEs(Ts, semiclassicals[0], deltaG, *params)
	return least_squares(resids, initGuess, jac='3-point', bounds = np.array([[0, 0],[np.inf, np.inf]]))

#Fluorene
Ts = np.array([323, 296, 273, 263, 239])
expKIEs = np.array([3.6, 3.57, 3.9, 4.1, 4.0])
deltaG = -3

#DHA
#Ts = np.array([323, 303, 283])
#expKIEs = np.array([11.4, 16, 26]) #DHA Kinetics
#deltaG = -7

semi_DEa = np.linspace(0, 1.4, 15)
semi_DlnA = np.array([-0.69, 0, 0.35])

solutions = np.zeros((15,3,2))

for j in range(len(semi_DlnA)):
	init = [12, 2000]
	for i in range(len(semi_DEa)):
		solutions[i,j] = fitKIEs(Ts, expKIEs, deltaG, init, [semi_DEa[i], semi_DlnA[j]]).x
		init = solutions[i,j]
		print(semi_DEa[i], semi_DlnA[j], solutions[i,j])


fig, axs = plt.subplots(1,2, figsize=(5,2.2), constrained_layout=True)
colors = ['r', 'purple', 'b']

for i in range(3):
	axs[0].plot(semi_DEa, solutions[:,i,0], color=colors[i])#, label="ln(A$_{{H}}$/A$_{{D}}$)$_{{S}}$ = {:>5.2f}".format(semi_DlnA[i]))
	axs[1].plot(semi_DEa, solutions[:,i,1], color=colors[i], label="ln(A$_{{H}}$/A$_{{D}}$)$_{{S}}$ = {:>5.2f}".format(semi_DlnA[i]))

axs[1].legend(borderpad=0, borderaxespad=0.5)


axs[0].set_xlim(0, 1.4)
axs[0].set_xticks([0, 0.7, 1.4])
axs[0].set_xticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.8, 0.9, 1.0, 1.1, 1.2, 1.3], minor=True)

axs[1].set_xlim(0, 1.4)
axs[1].set_xticks([0, 0.7, 1.4])
axs[1].set_xticks([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.8, 0.9, 1.0, 1.1, 1.2, 1.3], minor=True)

#Fluorene
axs[0].set_ylim(0, 20)
axs[0].set_yticks([0, 5, 10, 15, 20])
axs[0].set_yticks([1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, 16, 17, 18, 19], minor=True)

axs[1].set_ylim(0, 12000)
axs[1].set_yticks([0, 4000, 8000, 12000])
axs[1].set_yticks([1000, 2000, 3000, 5000, 6000, 7000, 9000, 10000, 11000], minor=True)

#DHA
#axs[0].set_ylim(5, 20, 25)
#axs[0].set_yticks([5, 10, 15, 20, 25])
#axs[0].set_yticks([6, 7, 8, 9, 11, 12, 13, 14, 16, 17, 18, 19, 21, 22, 23, 24], minor=True)

#axs[1].set_ylim(0, 4000)
#axs[1].set_yticks([0, 1000, 2000, 3000, 4000])
#axs[1].set_yticks([250, 500, 750, 1250, 1500, 1750, 2250, 2500, 2750, 3250, 3500, 3750], minor=True)

axs[0].set_xlabel("(E$_{a}$(D)-E$_{a}$(H))$_{S}$ (kcal/mol)")
axs[1].set_xlabel("(E$_{a}$(D)-E$_{a}$(H))$_{S}$ (kcal/mol)")
axs[0].set_ylabel("Barrier Height (kcal/mol)")
axs[1].set_ylabel("Barrier Curvature (cm$^{-1}$)")

plt.show()