import Kinetics
from scipy.optimize import least_squares
import numpy as np
import matplotlib.pyplot as plt
plt.style.use("Figures.mplstyle")


k_PCET = 2.3
KIE_PCET = 4
subProps = np.array([0.5, 0, 0.5])
KIE_from_p = lambda p: ((1-p)*subProps[1]-2*p*subProps[2]) / (p*subProps[1]-2*(1-p)*subProps[0])

ks_Scramble = np.array([1e-1, 1e-10]) #1e-10 is the DHA number; 1e-4 is the toluene number
KIEs_Scramble = np.array([100, 100])
ks_Dimer = 10.**np.arange(0, 10)

p_Hs_sim = np.zeros((len(ks_Scramble), len(ks_Dimer)))
KIEs_sim = np.zeros((len(ks_Scramble), len(ks_Dimer)))

for i in range(len(ks_Scramble)):
	for j in range(len(ks_Dimer)):
		k_Scramble = ks_Scramble[i]
		KIE_Scramble = KIEs_Scramble[i]
		k_Dimer = ks_Dimer[j]
		
		rxn = Kinetics.Reaction()

		#C-H activation reactions
		rxn.parseRate("CoIIIO SHH -{}-> CoIIIOH SH".format(k_PCET))
		rxn.parseRate("CoIIIO SHD -{}-> CoIIIOH SD".format(k_PCET/2))
		rxn.parseRate("CoIIIO SHD -{}-> CoIIIOH SH".format(k_PCET/KIE_PCET/2))
		rxn.parseRate("CoIIIO SDD -{}-> CoIIIOH SD".format(k_PCET/KIE_PCET))

		#Scrambling reactions
		rxn.parseRate("SHD SH -{}-> SD SHH".format(k_Scramble/2))
		rxn.parseRate("SDD SH -{}-> SD SHD".format(k_Scramble/KIE_Scramble))
		rxn.parseRate("SHH SD -{}-> SH SHD".format(k_Scramble))
		rxn.parseRate("SHD SD -{}-> SH SDD".format(k_Scramble/KIE_Scramble/2))
		
		#Dimerization reactions
		rxn.parseRate("SH SH -{}-> S2H2".format(k_Dimer))
		rxn.parseRate("SH SD -{}-> S2HD".format(2*k_Dimer))
		rxn.parseRate("SD SD -{}-> S2D2".format(k_Dimer))

		#Initial concentrations and integrate
		rxn.C0["CoIIIO"] = 0.00125
		rxn.C0["SHH"] = 0.025*subProps[0]
		rxn.C0["SHD"] = 0.025*subProps[1]
		rxn.C0["SDD"] = 0.025*subProps[2]
		rxn.integrate(t_final = 1000)
		
		if rxn.getConcentration(["CoIIIO"])[0,-1] > 0.00000125:
			print("Need longer integration time")
		int_ps = rxn.getConcentration(["S2H2", "S2HD", "S2D2"])[:,-1]
		int_ps = int_ps/np.sum(int_ps)
		p_Hs_sim[i,j] = np.array([1, 0.5, 0]) @ int_ps
		KIEs_sim[i,j] = KIE_from_p(p_Hs_sim[i,j])
		print(k_Dimer, rxn.getConcentration(["SH"]).max(), rxn.getConcentration(["SD"]).max(), KIEs_sim[i,j])
		#fig, axs = plt.subplots()
		#axs.plot(rxn.sol.t, rxn.getConcentration(["CoIIIO", "SHH", "SH", "S2H2"]).T)
		#plt.show()
		
	print()	

fig, axs = plt.subplots(figsize=(3.25, 2.9), constrained_layout=True)
styles = ['k-', 'r-']
for i in range(len(ks_Scramble)):
	axs.plot(np.log10(ks_Dimer), np.log(KIEs_sim[i]), styles[i], label="$\\it{{k}}$$_{{Scramble}}$ = {:.1e} M$^{{-1}}$s$^{{-1}}$\nKIE$_{{Scramble}}$ = {:>3}".format(ks_Scramble[i], KIEs_Scramble[i]))
axs.legend()

print(KIEs_sim)

axs.set_xlim(-1, 11)
axs.set_xticks(np.arange(-1, 13, 2))
axs.set_xticklabels(np.arange(-1,13,2))
axs.set_xticks(np.array([-0.5, 0, 0.5, 1.5, 2, 2.5, 3.5, 4, 4.5, 5.5, 6, 6.5, 7.5, 8, 8.5, 9.5, 10, 10.5]), minor=True)
axs.set_xlabel("$\\it{k}$$_{Dimer}$ (M$^{-1}$s$^{-1}$, log scale)")

axs.set_ylim(1, np.log(5))
axs.set_yticks(np.log(np.array([3,4,5,6])))
axs.set_yticklabels([3,4,5,6])
axs.set_yticks(np.log(np.array([3.25, 3.5, 3.75, 4.25, 4.5, 4.75, 5.25, 5.5, 5.75])), minor=True)
axs.set_ylabel("Apparent KIE (log scale)")

plt.show()
